
# q-expansion of newform 750.4.a.d, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 750
weight = 4
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-2, 0], [3, 0], [0, 0], [4, -9], [-7, -16], [-1, 4], [-20, 79], [-121, 70], [15, -20], [87, -82], [-138, 87], [-279, 76], [-230, -17], [155, -249], [273, 64], [35, -161], [-318, -119], [-546, -63], [48, 328], [-281, 457], [431, -485], [-264, -438], [766, -5], [51, -1028], [404, -425], [-376, 268], [-1237, 675], [-439, 745], [-471, 212], [348, -294], [-1223, 306], [-1568, 1271], [291, -1078], [-477, 192], [719, 884], [-688, -858], [-2190, -287], [1883, -1650], [-290, 760], [-68, -2521], [-403, -2312], [-2182, 12], [-199, 1990], [-4229, 1026], [-111, 2838], [-1421, 756], [1404, -3108], [-4471, 22], [-2710, 4442], [3072, 1206], [-987, -1799], [5574, 459], [5302, -1330], [-309, 2145], [-4213, 4454], [3404, -6493], [1954, -151], [411, 4151], [-681, 655], [5141, -6851], [2921, 510], [6000, -1036], [-645, 3180], [3027, -3969], [-1886, -1941], [4855, 512], [3373, -4587], [-540, -2076], [9529, -595], [6365, 645], [2702, 397], [-4678, 3104], [-2355, 2793], [-831, 213], [6538, -2323], [7126, -8197], [-2729, 6775], [3012, -6512], [-4499, 6046], [-5244, 12286], [-13609, 4673], [5058, -1256], [-6017, -1256], [-11042, 4860], [14853, 423], [-3220, 2347], [8145, 1917], [6470, -14571], [5043, -5056], [5294, -7167], [4209, 8793], [7476, -13014], [-10134, 4596], [3961, -12289], [2315, 1128], [16713, -5607], [-18780, 5322], [-10797, 16945], [2340, 3800], [14352, 4341], [-5100, -10527], [4161, -13815], [-18835, 8453], [-3619, -9578], [-11475, 9205], [-8335, 11088], [17497, 658], [-11039, 11068], [10576, -1789], [-10873, 9312], [23424, 2436], [-9377, -2334], [3373, -406], [-3432, -13684], [-1999, 1812], [-20158, 9146], [10494, 3268], [2352, -5753], [-10222, 8062], [-21011, -2384], [-8704, -14729], [-13364, 11204], [-15004, 3014], [14107, -6266], [21855, -6252], [-25830, 8938], [-2449, -8618], [-27611, 6802], [31515, -5442], [7532, 12904], [3014, -939], [13454, -20162], [25776, 7293], [17763, 5202], [12585, -13434], [-3244, 9864], [-12485, 23732], [14793, 3053], [25769, -22513], [15586, -24899], [10471, -8265], [3149, 24098], [1882, 19976], [-10096, 18095], [-11888, 4010], [9057, -1582], [23967, 250], [19697, -23341], [-15469, 24936], [-12537, 20693], [19251, -30084], [33522, -12282], [18039, -37722], [-18609, -9402], [15331, 17324], [-26346, 25701], [22199, -13031], [-8594, -21965], [16954, 18666], [-3162, 23475], [-15795, 5274], [-33117, -3085], [-23747, 43658], [23984, -34790], [-29905, -9193], [-13713, 12297], [29325, -29357], [-2153, -32940], [-11007, 29195], [-53541, 13348], [1117, 16795], [14113, -13018], [38123, -29443], [-32363, 54321], [-9539, -14754], [-2475, 25824], [1927, -30928], [34366, -1178], [13674, -45321], [-34863, 35448], [-30775, 12462], [-23482, -2695], [-9516, 32922], [-29111, -24904], [-7159, 29303], [11333, 746], [-57538, 11919], [21864, -5055], [-18774, 2286], [13675, -19913], [6664, -37615], [19681, 5170], [14847, -7345], [-43377, -5956], [35608, -32450], [-48803, 16247], [34000, -5650], [3896, -39828], [-4796, 18292], [-5144, 15284], [18149, 38831], [-19409, 56399], [-20040, 5202], [-14963, -10942], [-35380, -2980], [-18441, 7577], [5211, 13103], [19378, -35663], [-24109, -25007], [-51192, 9863], [-54825, 35837], [9036, -29678], [38223, -26662], [-19014, -19512], [37668, -20400], [-13689, -26730], [36506, -7847], [-44175, 72558], [-7889, 23899], [10245, -4664], [17701, -14617], [24209, -71508], [26184, -18678], [-31497, -13638], [-49992, 74197], [-34106, 34278], [-1834, -56626], [-38041, 28132], [32955, -35725], [-24098, -22088], [81492, -26997], [25008, -47686], [63494, -10625], [55439, -24907], [26104, 15264], [-40176, 32791], [-32670, -7060], [10430, 23806], [-28075, -8056], [-39600, 46969], [10431, -42806], [-70930, 77355], [21147, 45809], [-50829, -40418], [30359, 26663], [-60005, -5249], [-104293, -912], [-53561, -35875], [-8108, -9918], [44290, -80264], [-3240, -44463], [7857, -7557], [7586, -1534], [70519, -34005], [21363, -19160], [19617, -958], [-60363, 6998], [26841, 28826], [89808, -10528], [-50030, 48398], [-18816, -29378], [61870, -8708], [-7195, 31956], [-44789, 106629], [14482, 58084], [4763, -56784], [21721, -9952], [-10410, -45136], [-34621, 75306], [-46234, 47807], [-62154, 25485], [-21835, -54660], [44164, -1974], [51804, -93228], [69867, -53035], [-94822, 14502], [-55232, -48640], [-93867, -29352], [-18081, -22502], [-81163, -18974], [127473, -8589], [-33526, 79387], [109179, -8241], [-74032, 26884], [9813, -84712], [-73562, 907], [-74779, -5013], [-57432, 42567], [-14737, 2448], [120193, 17822], [96576, -65802], [-36420, -46200], [111370, -88931], [79914, -14758], [-37160, 71906], [36109, -18209], [66365, -51288], [131311, 535], [-5412, 101414], [22542, 61519], [-51225, 108410], [47367, 44489], [-8911, 31717], [68371, -33320], [13218, 88170], [-95653, 106527], [122835, -50298], [-119090, 93087], [2175, 88664], [17918, -77967], [-104829, 91335], [106271, -77969], [-11061, 8634], [-75670, 139027], [-2928, 2268], [-111598, 70509], [-100186, 48161], [-8706, -58560], [42609, -87798], [-10791, 88622], [70557, -13301], [-82473, 87399], [128844, 14029], [99790, -106444], [-58884, 15282], [20595, -109776], [96726, -54367], [-12715, -396], [-71581, 167314], [85689, 6156], [5398, -74037], [148269, -5784], [-3999, -31031], [129239, -10048], [-39879, 120639], [-3954, 18542], [-124101, 65940], [-36621, -32305], [-138971, -30885], [-108091, 35052], [152787, -41190], [19218, 84711], [-53047, 122255], [-93206, 43992], [123586, -43259], [14288, -23104], [-18774, -52146], [29215, -139891], [78621, -153531], [120597, -29361], [52492, -115014], [-131755, 62725], [-108996, 147516], [62863, 52176], [25683, 104031], [114088, -124907], [20166, 131184], [173951, 15452], [-41366, 113438], [-64844, 38335], [107555, -6600], [-87029, 116732], [-52959, -66312], [-80022, 87864], [-179968, 4894], [-63246, -35680], [61178, -77611], [-135489, 66236], [-53373, 70368], [-34488, -106697], [19864, -58161], [-76252, -26879], [-49514, 67534], [-118924, 167458], [-54317, 52311], [28565, -19208], [-82116, 8178], [-90336, -83274], [-125416, 52613], [6554, 97116], [-112496, 142843], [-220416, 51088], [-52385, 34488], [-126672, -14694], [-27774, 23217], [-121938, 203298], [-72689, 136809], [130308, -138347], [-89712, 61761], [89623, -160142], [-217663, -7998], [189177, -13830], [100844, -115367], [-309, 34695], [-40407, -17168], [226656, -49014], [22763, -88775], [-7883, -1838], [-29915, -20635], [-130287, 134246], [-119055, 89097], [50827, -123229], [5547, -56881], [-240602, 49735], [165712, -81254], [-50311, 26050], [-32220, 145086], [-212883, 26184], [6355, -81084], [-30021, -36872], [10061, 15300], [6075, 15354], [-32973, -157494], [86357, 119544], [65926, 70316], [1953, -90393], [45131, 8090], [-81600, 32988], [-238704, 81440], [-185303, 88132], [-239149, 18479], [32071, 96672], [-105494, -22169], [24261, 36052], [-107305, -129303], [-12077, 24062], [-102540, 109796], [-135740, -46678], [27123, 172844], [-159364, 127391]]
